@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
 // A RTSP server
 // Implementation
 
@@ -31,16 +31,14 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #endif
 #include <time.h> // for "strftime()" and "gmtime()"
 
-// Leave the following commented out for now, because - for some still-unknown
-// reason - including a "rtptime=" field in the "RTPInfo:" header confuses
-// QuickTime Player, when we seek within a stream.
-//#define RTPINFO_INCLUDE_RTPTIME 1
+#define RTPINFO_INCLUDE_RTPTIME 1
 
 ////////// RTSPServer //////////
 
 RTSPServer*
 RTSPServer::createNew(UsageEnvironment& env, Port ourPort,
-		      UserAuthenticationDatabase* authDatabase) {
+		      UserAuthenticationDatabase* authDatabase,
+		      unsigned reclamationTestSeconds) {
   int ourSocket = -1;
   RTSPServer* newServer = NULL;
 
@@ -48,10 +46,11 @@ RTSPServer::createNew(UsageEnvironment& env, Port ourPort,
     int ourSocket = setUpOurSocket(env, ourPort);
     if (ourSocket == -1) break;
 
-    return new RTSPServer(env, ourSocket, ourPort, authDatabase);
+    return new RTSPServer(env, ourSocket, ourPort, authDatabase,
+			  reclamationTestSeconds);
   } while (0);
 
-  if (ourSocket != -1) ::_close(ourSocket);
+  if (ourSocket != -1) ::closeSocket(ourSocket);
   delete newServer;
   return NULL;
 }
@@ -85,6 +84,10 @@ void RTSPServer::addServerMediaSession(ServerMediaSession* serverMediaSession) {
   removeServerMediaSession(existingSession); // if any
 }
 
+ServerMediaSession* RTSPServer::lookupServerMediaSession(char const* streamName) {
+  return (ServerMediaSession*)(fServerMediaSessions->Lookup(streamName));
+}
+
 void RTSPServer::removeServerMediaSession(ServerMediaSession* serverMediaSession) {
   if (serverMediaSession == NULL) return;
 
@@ -97,7 +100,7 @@ void RTSPServer::removeServerMediaSession(ServerMediaSession* serverMediaSession
 }
 
 void RTSPServer::removeServerMediaSession(char const* streamName) {
-  removeServerMediaSession((ServerMediaSession*)(fServerMediaSessions->Lookup(streamName)));
+  removeServerMediaSession(lookupServerMediaSession(streamName));
 }
 
 char* RTSPServer
@@ -154,16 +157,17 @@ int RTSPServer::setUpOurSocket(UsageEnvironment& env, Port& ourPort) {
     return ourSocket;
   } while (0);  
 
-  if (ourSocket != -1) ::_close(ourSocket);
+  if (ourSocket != -1) ::closeSocket(ourSocket);
   return -1;
 }
 
 RTSPServer::RTSPServer(UsageEnvironment& env,
 		       int ourSocket, Port ourPort,
-		       UserAuthenticationDatabase* authDatabase)
+		       UserAuthenticationDatabase* authDatabase,
+		       unsigned reclamationTestSeconds)
   : Medium(env),
     fServerSocket(ourSocket), fServerPort(ourPort),
-    fAuthDB(authDatabase),
+    fAuthDB(authDatabase), fReclamationTestSeconds(reclamationTestSeconds),
     fServerMediaSessions(HashTable::create(STRING_HASH_KEYS)), 
     fSessionIdCounter(0) {
 #ifdef USE_SIGNALS
@@ -182,7 +186,7 @@ RTSPServer::~RTSPServer() {
   // Turn off background read handling:
   envir().taskScheduler().turnOffBackgroundReadHandling(fServerSocket);
 
-  ::_close(fServerSocket);
+  ::closeSocket(fServerSocket);
 
   // Remove all server media sessions (they'll get deleted when they're finished):
   while (1) {
@@ -224,8 +228,6 @@ void RTSPServer::incomingConnectionHandler1() {
 #endif
 
   // Create a new object for this RTSP session:
-  // (Later, we need to do some garbage collection on sessions that #####
-  //  aren't closed down via TEARDOWN) #####
   new RTSPClientSession(*this, ++fSessionIdCounter,
 			clientSocket, clientAddr);
 }
@@ -239,18 +241,22 @@ RTSPServer::RTSPClientSession
   : fOurServer(ourServer), fOurSessionId(sessionId),
     fOurServerMediaSession(NULL),
     fClientSocket(clientSocket), fClientAddr(clientAddr),
-    fSessionIsActive(True), fStreamAfterSETUP(False),
+    fLivenessCheckTask(NULL), fSessionIsActive(True), fStreamAfterSETUP(False),
     fTCPStreamIdCount(0), fNumStreamStates(0), fStreamStates(NULL) {
   // Arrange to handle incoming requests:
   envir().taskScheduler().turnOnBackgroundReadHandling(fClientSocket,
      (TaskScheduler::BackgroundHandlerProc*)&incomingRequestHandler, this);
+  noteLiveness();
 }
 
 RTSPServer::RTSPClientSession::~RTSPClientSession() {
+  // Turn off any liveness checking:
+  envir().taskScheduler().unscheduleDelayedTask(fLivenessCheckTask);
+
   // Turn off background read handling:
   envir().taskScheduler().turnOffBackgroundReadHandling(fClientSocket);
 
-  ::_close(fClientSocket);
+  ::closeSocket(fClientSocket);
 
   reclaimStreamStates();
 
@@ -281,6 +287,8 @@ void RTSPServer::RTSPClientSession
 }
 
 void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
+  noteLiveness();
+
   struct sockaddr_in dummy; // 'from' address, meaningless in this case
   int bytesLeft = sizeof fBuffer;
   int totalBytes = 0;
@@ -364,9 +372,9 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
   }
     
 #ifdef DEBUG
-  fprintf(stderr, "sending response: %s", fBuffer);
+  fprintf(stderr, "sending response: %s", fResponseBuffer);
 #endif
-  send(fClientSocket, (char const*)fBuffer, strlen((char*)fBuffer), 0);
+  send(fClientSocket, (char const*)fResponseBuffer, strlen((char*)fResponseBuffer), 0);
 
   if (strcmp(cmdName, "SETUP") == 0 && fStreamAfterSETUP) {
     // The client has asked for streaming to commence now, rather than after a
@@ -382,8 +390,28 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
 // Generate a "Date:" header for use in a RTSP response:
 static char const* dateHeader() {
   static char buf[200];
+#if !defined(_WIN32_WCE)
   time_t tt = time(NULL);
   strftime(buf, sizeof buf, "Date: %a, %b %d %Y %H:%M:%S GMT\r\n", gmtime(&tt));
+#else
+  // WinCE apparently doesn't have "time()", "strftime()", or "gmtime()",
+  // so generate the "Date:" header a different, WinCE-specific way.
+  // (Thanks to Pierre l'Hussiez for this code)
+  SYSTEMTIME SystemTime;
+  GetSystemTime(&SystemTime);
+  WCHAR dateFormat[] = L"ddd, MMM dd yyyy";
+  WCHAR timeFormat[] = L"HH:mm:ss GMT\r\n";
+  WCHAR inBuf[200];
+  DWORD locale = LOCALE_NEUTRAL;
+  
+  int ret = GetDateFormat(locale, 0, &SystemTime,
+			  (LPTSTR)dateFormat, (LPTSTR)inBuf, sizeof inBuf);
+  inBuf[ret - 1] = ' ';
+  ret = GetTimeFormat(locale, 0, &SystemTime,
+		      (LPTSTR)timeFormat,
+		      (LPTSTR)inBuf + ret, (sizeof inBuf) - ret);
+  wcstombs(buf, inBuf, wcslen(inBuf));
+#endif
   return buf;
 }
 
@@ -392,35 +420,35 @@ static char const* allowedCommandNames
 
 void RTSPServer::RTSPClientSession::handleCmd_bad(char const* /*cseq*/) {
   // Don't do anything with "cseq", because it might be nonsense
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 400 Bad Request\r\n%sAllow: %s\r\n\r\n",
 	   dateHeader(), allowedCommandNames);
   fSessionIsActive = False; // triggers deletion of ourself after responding
 }
 
 void RTSPServer::RTSPClientSession::handleCmd_notSupported(char const* cseq) {
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 405 Method Not Allowed\r\nCSeq: %s\r\n%sAllow: %s\r\n\r\n",
 	   cseq, dateHeader(), allowedCommandNames);
   fSessionIsActive = False; // triggers deletion of ourself after responding
 }
 
 void RTSPServer::RTSPClientSession::handleCmd_notFound(char const* cseq) {
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 404 Stream Not Found\r\nCSeq: %s\r\n%s\r\n",
 	   cseq, dateHeader());
   fSessionIsActive = False; // triggers deletion of ourself after responding
 }
 
 void RTSPServer::RTSPClientSession::handleCmd_unsupportedTransport(char const* cseq) {
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 461 Unsupported Transport\r\nCSeq: %s\r\n%s\r\n",
 	   cseq, dateHeader());
   fSessionIsActive = False; // triggers deletion of ourself after responding
 }
 
 void RTSPServer::RTSPClientSession::handleCmd_OPTIONS(char const* cseq) {
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sPublic: %s\r\n\r\n",
 	   cseq, dateHeader(), allowedCommandNames);
 }
@@ -438,8 +466,7 @@ void RTSPServer::RTSPClientSession
 
     // Begin by looking up the "ServerMediaSession" object for the
     // specified "urlSuffix":
-    ServerMediaSession* session
-      = (ServerMediaSession*)(fOurServer.fServerMediaSessions->Lookup(urlSuffix));
+    ServerMediaSession* session = fOurServer.lookupServerMediaSession(urlSuffix);
     if (session == NULL) {
       handleCmd_notFound(cseq);
       break;
@@ -450,7 +477,7 @@ void RTSPServer::RTSPClientSession
     if (sdpDescription == NULL) {
       // This usually means that a file name that was specified for a
       // "ServerMediaSubsession" does not exist.
-      snprintf((char*)fBuffer, sizeof fBuffer,
+      snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	       "RTSP/1.0 404 File Not Found, Or In Incorrect Format\r\n"
 	       "CSeq: %s\r\n"
 	       "%s\r\n",
@@ -460,12 +487,12 @@ void RTSPServer::RTSPClientSession
     }
     unsigned sdpDescriptionSize = strlen(sdpDescription);
 
-    // Also, generate out RTSP URL, for the "Content-Base:" header
+    // Also, generate our RTSP URL, for the "Content-Base:" header
     // (which is necessary to ensure that the correct URL gets used in
     // subsequent "SETUP" requests).
     rtspURL = fOurServer.rtspURL(session);
 
-    snprintf((char*)fBuffer, sizeof fBuffer,
+    snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	     "RTSP/1.0 200 OK\r\nCSeq: %s\r\n"
 	     "%s"
 	     "Content-Base: %s/\r\n"
@@ -599,13 +626,12 @@ void RTSPServer::RTSPClientSession
 
     // Look up the "ServerMediaSession" object for the specified stream:
     if (streamName[0] != '\0' ||
-	fOurServer.fServerMediaSessions->Lookup("") != NULL) { // normal case
+	fOurServer.lookupServerMediaSession("") != NULL) { // normal case
     } else { // weird case: there was no track id in the URL
       streamName = urlSuffix;
       trackId = NULL;
     }
-    fOurServerMediaSession = (ServerMediaSession*)
-      (fOurServer.fServerMediaSessions->Lookup(streamName));
+    fOurServerMediaSession = fOurServer.lookupServerMediaSession(streamName);
     if (fOurServerMediaSession == NULL) {
       handleCmd_notFound(cseq);
       return;
@@ -630,7 +656,7 @@ void RTSPServer::RTSPClientSession
   // Look up information for the specified subsession (track):
   ServerMediaSubsession* subsession = NULL;
   unsigned streamNum;
-  if (trackId != NULL) { // normal case
+  if (trackId != NULL && trackId[0] != '\0') { // normal case
     for (streamNum = 0; streamNum < fNumStreamStates; ++streamNum) {
       subsession = fStreamStates[streamNum].subsession;
       if (subsession != NULL && strcmp(trackId, subsession->trackId()) == 0) break;
@@ -712,7 +738,7 @@ void RTSPServer::RTSPClientSession
       handleCmd_unsupportedTransport(cseq);
       return;
     }
-    snprintf((char*)fBuffer, sizeof fBuffer,
+    snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	     "RTSP/1.0 200 OK\r\n"
 	     "CSeq: %s\r\n"
 	     "%s"
@@ -725,7 +751,7 @@ void RTSPServer::RTSPClientSession
   } else {
     switch (streamingMode) {
     case RTP_UDP: {
-      snprintf((char*)fBuffer, sizeof fBuffer,
+      snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	       "RTSP/1.0 200 OK\r\n"
 	       "CSeq: %s\r\n"
 	       "%s"
@@ -738,7 +764,7 @@ void RTSPServer::RTSPClientSession
       break;
     }
     case RTP_TCP: {
-      snprintf((char*)fBuffer, sizeof fBuffer,
+      snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	       "RTSP/1.0 200 OK\r\n"
 	       "CSeq: %s\r\n"
 	       "%s"
@@ -751,7 +777,7 @@ void RTSPServer::RTSPClientSession
       break;
     }
     case RAW_UDP: {
-      snprintf((char*)fBuffer, sizeof fBuffer,
+      snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	       "RTSP/1.0 200 OK\r\n"
 	       "CSeq: %s\r\n"
 	       "%s"
@@ -818,7 +844,7 @@ void RTSPServer::RTSPClientSession
 
 void RTSPServer::RTSPClientSession
 ::handleCmd_TEARDOWN(ServerMediaSubsession* /*subsession*/, char const* cseq) {
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%s\r\n",
 	   cseq, dateHeader());
   fSessionIsActive = False; // triggers deletion of ourself after responding
@@ -944,6 +970,8 @@ void RTSPServer::RTSPClientSession
       unsigned rtpTimestamp = 0;
       fStreamStates[i].subsession->startStream(fOurSessionId,
 					       fStreamStates[i].streamToken,
+					       (TaskFunc*)noteClientLiveness,
+					       this,
 					       rtpSeqNum, rtpTimestamp);
       const char *urlSuffix = fStreamStates[i].subsession->trackId();
       char* prevRTPInfo = rtpInfo;
@@ -979,7 +1007,7 @@ void RTSPServer::RTSPClientSession
   }
 
   // Fill in the response:
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 200 OK\r\n"
 	   "CSeq: %s\r\n"
 	   "%s"
@@ -1006,17 +1034,17 @@ void RTSPServer::RTSPClientSession
 					       fStreamStates[i].streamToken);
     }
   }
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %d\r\n\r\n",
 	   cseq, dateHeader(), fOurSessionId);
 }
 
 void RTSPServer::RTSPClientSession
-::handleCmd_GET_PARAMETER(ServerMediaSubsession* /*subsession*/, char const* cseq,
+::handleCmd_GET_PARAMETER(ServerMediaSubsession* subsession, char const* cseq,
 			  char const* /*fullRequestStr*/) {
   // We implement "GET_PARAMETER" just as a 'keep alive',
   // and send back an empty response:
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %d\r\n\r\n",
 	   cseq, dateHeader(), fOurSessionId);
 }
@@ -1120,7 +1148,7 @@ Boolean RTSPServer::RTSPClientSession
   // If we get here, there was some kind of authentication failure.
   // Send back a "401 Unauthorized" response, with a new random nonce:
   fCurrentAuthenticator.setRealmAndRandomNonce(fOurServer.fAuthDB->realm());
-  snprintf((char*)fBuffer, sizeof fBuffer,
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 401 Unauthorized\r\n"
 	   "CSeq: %s\r\n"
 	   "%s"
@@ -1217,13 +1245,15 @@ RTSPServer::RTSPClientSession
   }
   if (!parseSucceeded) return False;
 
-  // Look for "CSeq: ", then read everything up to the next \r as 'CSeq':
+  // Look for "CSeq:", skip whitespace,
+  // then read everything up to the next \r or \n as 'CSeq':
   parseSucceeded = False;
-  for (j = i; j < reqStrSize-6; ++j) {
+  for (j = i; j < reqStrSize-5; ++j) {
     if (reqStr[j] == 'C' && reqStr[j+1] == 'S' && reqStr[j+2] == 'e' &&
-	reqStr[j+3] == 'q' && reqStr[j+4] == ':' && reqStr[j+5] == ' ') {
-      j += 6;
+	reqStr[j+3] == 'q' && reqStr[j+4] == ':') {
+      j += 5;
       unsigned n;
+      while (j < reqStrSize && (reqStr[j] ==  ' ' || reqStr[j] == '\t')) ++j;
       for (n = 0; n < resultCSeqMaxSize-1 && j < reqStrSize; ++n,++j) {
 	char c = reqStr[j];
 	if (c == '\r' || c == '\n') {
@@ -1242,13 +1272,37 @@ RTSPServer::RTSPClientSession
   return True;
 }
 
+void RTSPServer::RTSPClientSession::noteLiveness() {
+  if (fOurServer.fReclamationTestSeconds > 0) {
+    envir().taskScheduler()
+      .rescheduleDelayedTask(fLivenessCheckTask,
+			     fOurServer.fReclamationTestSeconds*1000000,
+			     (TaskFunc*)livenessTimeoutTask, this);
+  }
+}
+
+void RTSPServer::RTSPClientSession
+::noteClientLiveness(RTSPClientSession* clientSession) {
+  clientSession->noteLiveness();
+}
+
+void RTSPServer::RTSPClientSession
+::livenessTimeoutTask(RTSPClientSession* clientSession) {
+  // If this gets called, the client session is assumed to have timed out,
+  // so delete it:
+#ifdef DEBUG
+  fprintf(stderr, "RTSP client session from %s has timed out (due to inactivity)\n", our_inet_ntoa(clientSession->fClientAddr.sin_addr));
+#endif
+  delete clientSession;
+}
+
 
 ////////// UserAuthenticationDatabase implementation //////////
 
 UserAuthenticationDatabase::UserAuthenticationDatabase(char const* realm,
 						       Boolean passwordsAreMD5)
   : fTable(HashTable::create(STRING_HASH_KEYS)),
-    fRealm(strDup(realm == NULL ? "LIVE.COM Streaming Media" : realm)),
+    fRealm(strDup(realm == NULL ? "LIVE555 Streaming Media" : realm)),
     fPasswordsAreMD5(passwordsAreMD5) {
 }
 
